% Octave routine to analyse Spectravue .csv file for half drift scan - DL3WDG 19 April 2024

clear all
close all
%clf
clc
pkg load data-smoothing
scrsz=get(0,'Screensize');
hpix=scrsz(3); % number of horizontal screen pixels
vpix=scrsz(4); % number of vertical screen pixels

##################################################################################
### Grafted in KA1GT "pre-processing" routine to determine data collection rate (points per second)
### find index values and strength for left sidelope and right sidelobe
### and define the index points for the start and end of the valid data.
### variables should not clash with the rest of the code (I hope)
##################################################################################

#  looks for data rate in SpectraVue continuum data - KA1GT
#  NOTE - Requires data/time format to be YYYY-MM-DD HH:MM:SS e.g. 2024-04-20 16:30:05. The deliniters don't matter, but the position if the digits does.
#  A data recording time of at least 21s is assumed, since timing is measured ocer a 20s period.

[filename,pathname]=uigetfile('*.csv','Select SepctaVue csv file'); #select input .csv file
File_Path=strcat(pathname,filename) # create full path to file

fid = fopen (File_Path); # open the file for reading/writing
################################# new code for <10 hours UTC time###################################
CSV_Line = fgetl(fid) ;# get first line of csv file date time, level, level
[Date_Time,rest_of_line] = strtok(CSV_Line, ",") ;# date time is before first comma
[CSV_Levels,CSV_Time] = strtok(Date_Time); ## Split at space
CSV_Time=strtrim(CSV_Time) ; ## remove leading space
        if (length(CSV_Time) ==7 ) # if hours <10
                CSV_Time = strcat("0",CSV_Time) # add a leading sero
        endif
Hours = substr(CSV_Time,1,2);
Mins = substr(CSV_Time,4,2);
Secs = substr(CSV_Time,7,2);
Time_spm =  (str2double (Hours)*60*60) +  (str2double (Mins)*60) +  (str2double (Secs)) ; #This is the time in seconds after midnight UTC. If scan includes mignight, won't work!!

#Time_spm is seconds after midnight UTC of the first data point. Don't make a scan that crosses 00:00:00 UTC or the time calculation will fail!
#Now look for first entry with Tx = T +1

NextTime=Time_spm; # Initialize variable Time in Seconds Past Midnight when the file starts
Index_Count=0;

while NextTime == Time_spm  #read lines untill the time changes (i.e. seconds change). This will be the start of the timing count
    Index_Count = Index_Count+1;
    CSV_Line = fgetl(fid) ;# get first line of csv file date time, level, level
[Date_Time,rest_of_line] = strtok(CSV_Line, ","); # date time is before first comma
[CSV_Levels,CSV_Time] = strtok(Date_Time) ;## Split at space
CSV_Time=strtrim(CSV_Time) ; ## remove leading space
        if (length(CSV_Time) ==7 );# if hours <10
                CSV_Time = strcat("0",CSV_Time) ;# add a leading sero
        endif
Hours = substr(CSV_Time,1,2);
Mins = substr(CSV_Time,4,2);
Secs = substr(CSV_Time,7,2);
NextTime =  (str2double (Hours)*60*60) +  (str2double (Mins)*60) +  (str2double (Secs))  #This is the time in seconds after midnight UTC. If scan includes mignight, won't work!!
endwhile

First_Time_Index = Index_Count + 1 #start of of the section in which to count data points

#First_Time_Index is the index of the first data point after the senonds change.  Now look for the index of the data at which the time has increased by 20 seconds

Time_spm2 = Time_spm+21 #look for end of time block which is 20s into the future, Could be longer, depending on file length and data rate, but 20s is probably safe.

while NextTime != Time_spm2
        Index_Count = Index_Count+1;
         CSV_Line = fgetl(fid) ;# get first line of csv file date time, level, level
        [Date_Time,rest_of_line] = strtok(CSV_Line, ",") ;# date time is before first comma
        [CSV_Levels,CSV_Time] = strtok(Date_Time) ;## Split at space
        CSV_Time=strtrim(CSV_Time) ; ## remove leading space
             if (length(CSV_Time) ==7 );# if hours <10
                CSV_Time = strcat("0",CSV_Time) ;# add a leading sero
             endif
        Hours = substr(CSV_Time,1,2);
        Mins = substr(CSV_Time,4,2);
        Secs = substr(CSV_Time,7,2);
        NextTime =  (str2double (Hours)*60*60) +  (str2double (Mins)*60) +  (str2double (Secs))  #This is the time in seconds after midnight UTC. If scan includes mignight, won't work!!
endwhile


Secong_Time_Index = Index_Count+1 # this is the sart point of data taken 20s after the start point.

fclose (fid); # close file, stop sequetial reading of file lines and reset index back to start for any future reads

# in 20s there were Last_time - First_Time_Index readings
Datapoints_per_Second = (Secong_Time_Index - First_Time_Index)/20

#-###################That's the end of the time and points per second calculation#########################

##### ATTEMPT TO LOOP BACK TO THIS POINT####KA1GT #############

 while 1==1  ## KA1GT ## loop forever (exit oportunity at end ########. Permits multiple analysis of file without reloading.


#Now plot the data (and the smoothed data, to check it's OK ) and locate the index numbers of required data points (start and syop of valid date, position of fide lobes)

csv_Data=csvread (File_Path);    #reads in csv data
Data_Length=length(csv_Data); #Number of data points

x_vector = [1:Data_Length]';  # Create an x axis vector for smoothing function. The ' transposes the vector from row to column

signal_dB = csv_Data(:,2);     # get a vector of the SpectraVue continuum values for use by the smoothing function

#Smooth the data to remove noise but keep original shape by using an appropriate value of Lambda. Higher lambda gives smoother curce put may not follow small changes
[smdata, lambda1] = regdatasmooth (x_vector, signal_dB, "lambda", 0.0000001); # Lambda adjusts the degree of smoothing. For this data 0.0000001 seems good

#[smdata2, lambda2] = regdatasmooth (x_vector, signal_dB); # let smoothing function find lambda, This slows things down significantly, but only for initial analysis.

figure(10); # Plot smothed data on top of raw data to check all looks good
hold on
plot(signal_dB, "linewidth",1,"color","k");
plot (smdata, "linewidth",2, "color", "r");
#plot (smdata2, "linewidth", 2, "color", "r");

########## ROUTINE TO GET NEW COORDINATES FROM A PLOT  ########

msgbox ("CLOSE THIS WINDOW AND THEN:\n\n1. Place mouse cursor over first data point and left click\n2. Place mouse cursor over first sidelobe peak and left click\n3. Place mouse cursor over second sidelobe peak and left click\n4. Place mouse cursor over last data point and left click");
[side_x, side_y] = ginput (4); # get 4 sets of coorinates before continuing

# Out of Bounds Checks.  Putting the sursor to the left of the data points  and to the right of the data points will not change data span.
if (side_x(1) <1)
  side_x(1) =1
endif
if (side_x(4) > Data_Length)
  side_x(4) = Data_Length
endif

Start_Data = round(side_x(1))
Left_Sidelobe = round(side_x(2))
Right_Sidelobe = round(side_x(3))
Stop_Data = round (side_x(4))

Left_Sidelobs_Index = round(side_x(2)) - round(side_x(1))
Right_Sidelobe_Index = round(side_x(3)) - round(side_x(1))

msg_line1 = strcat ("data points per second = ", num2str(Datapoints_per_Second));
msg_line2 = strcat ("New first data point index = ",num2str(Start_Data));
msg_line3 = strcat ("New left sidelobe index = ", num2str(Left_Sidelobs_Index));
msg_line4 = strcat ("New right sidelobe index = ", num2str(Right_Sidelobe_Index));
msg_line5 = strcat ("last data point index = ", num2str(Stop_Data));

waitfor(msgbox({msg_line1,msg_line2,msg_line3,msg_line4,msg_line5}));

#######################################################################################################################
################################################# END of KA1GT code ###################################################
#######################################################################################################################




 wfn=File_Path; #Passing KA1GT variable name to DL3WDG variable name.

 S1=csvread (wfn,0,1);  # reads in csv data ignoring troublesome time field. Creates a 2 column matrix
 S=S1(Start_Data:Stop_Data) ' ; #'#Now select the data to be used  into a vector.  ###KA1GT### I have no idea why this has to be transposed (  '  ).

Length=length(S);%no of lines of input data
Length


prompt = {'*Points/sec', '*lower sidelobe index','*upper sidelobe index','time1', 'az1', 'el1', 'time2','az2','el2','smoothing (0-3)'};

 dlg_title = 'Input parameters - DL3WDG';
   #defaultans = {'93.75','0.75','316','580','56.5',}; #############commented out KA1GT#######################
   %defaultans = {num2str(Datapoints_per_Second),'1.426',num2str(Left_Sidelobs_Index),num2str(Right_Sidelobe_Index),'47','1'};#########changed by KA1GT###############

   starttime = '11:04:09';
   startaz = '174.926';
   startel = '60.665';
   stoptime = '11:22:52';
   stopaz = '183.980';
   stopel = '60.690';

   defaultans = {num2str(Datapoints_per_Second),num2str(Left_Sidelobs_Index),num2str(Right_Sidelobe_Index),starttime,startaz,startel,stoptime,stopaz,stopel, '1',};


 answer = inputdlg(prompt,dlg_title,[1 11; 1 11; 1 11; 1 11; 1 11;1 11; 1 11; 1 11; 1 11;1 11],defaultans);

   pointspsec = str2num(answer{1});

   Lsidelobeindex= int32(str2num(answer{2}));
   Usidelobeindex= int32(str2num(answer{3}))
   time1=datevec(answer(4));
   az1 = str2num(answer{5});
   elevation = str2num(answer{6});
   time2=datevec(answer(7));
   az2=str2num(answer{8});
   elevation2 = str2num(answer{9});
 smoothing = (answer{10});



   timeH1=time1(4);
   timeM1=time1(5);
   timeS1=time1(6);
   timeaz1=(3600*timeH1)+(60*timeM1)+timeS1;

   timeH2=time2(4);
   timeM2=time2(5);
   timeS2=time2(6);
   timeaz2=(3600*timeH2)+(60*timeM2)+timeS2;

   aztimediff=timeaz2-timeaz1
   azdiff=az2-az1
   azrate=azdiff/aztimediff
   eldiff=elevation2-elevation
   elrate=eldiff/aztimediff

   interval = 1/pointspsec


h=figure; % GUI for plot
set(h,'Name','Full Drift Scan analysis by DL3WDG','NumberTitle','off','Menubar','Figure','Position',[round(0.04*hpix) round(0.42*vpix) round(0.44*hpix) round(0.44*vpix)])

# Change smoothing coefficient 1 is normally OK,  2 to 4 are smoother, but may niss rapidly changing data  ### KA1GT ###
if (str2num(smoothing)==1)
sc = 1E-7
elseif (str2num(smoothing)==2)
sc=1E-6
elseif(str2num(smoothing)==3)
sc=1e-5
elseif(str2num(smoothing)==4)
sc=1e-4
elseif(str2num(smoothing)==0)
sc=1e-8
else sc=1E-7 # default if invalid number is typed
endif

x_vector = [1:Length]'; #create and transpose vestor
[Ns, lambda] = regdatasmooth (x_vector, S, "lambda", sc); ### S is a vector, so no need to manupulate it???  ###KA1GT######

##################### Removed by KA!GT ####################
#N = S(:,1);%gets the first Spectravue column (the one we are interested in) ### I have no idea what ths does...N comes out as a double, not vector...?? ####KA1GT#####
#[Ns, lambda] = regdatasmooth (x_vector, N, "lambda", 0.0000001);
##########################################################

[peak,ipeak] = max(Ns)%peak is peak value and ipeak is its index

xaxis=0; %used for plotting
startnoise=0;
endnoise=0;
normnoise=0;
normnoise2=0;
%endnoise1=0;
%endnoise2=0;
avendnoise=0;
%avendnoise1=0;
%avendnoise1=0;



### KAIGT  ##### alternative to average 50 lowest values rather than last 50 values
               [sortedVals] = sort(Ns); # sort in ascending order
               lowavenoise = sum (sortedVals(1:50))/50 # average the lowest 50 points
               ## KA1GT  ### use lowavenoise rather then avendnoise as baseline for noise

avendnoise = lowavenoise




% this loop gets average noise level at end of data from 51 points
for i=(5:55)
startnoise=startnoise+Ns(i);
endfor
avstartnoise=startnoise/51%gets averaged noise level at start of test



normfact=(10^(peak/10)-10^(avendnoise/10))

for i=1:Length%looking in left hand part of data
normnoisefull(i,:)=(10^(Ns(i)/10)-10^(avendnoise/10))/normfact;%column vector of normalised noise data

endfor
xaxis=linspace(1,Length*1,Length); % generates the x-axix for the plots


for i=1:ipeak%looking in left hand part of data
normnoise(i,:)=(10^(Ns(i)/10)-10^(avstartnoise/10))/normfact;%column vector of normalised noise data
endfor


target = 0.15;%searching for point in normalised data to match value of 0.15 for Joachim tool
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L15=Index%is index of lower 0.15 match

target = 0.2;%searching for a specific value
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L2=Index%is index of lower 0.2 match

target = 0.3;%searching for a specific value
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L3=Index%is index of lower 0.3 match

target = 0.7;%searching for a specific value
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L7=Index%is index of lower 0.7 match

target = 0.8;%searching for a specific value
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L8=Index%is index of lower 0.8 match

target = 0.85;%searching for a specific value
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L85=Index%is index of lower 0.85 match

target = 0.5;%searching for a specific value (the -3dB point for crude HPBW)
Index=find(min(abs(normnoise-target))==abs(normnoise-target));%returns index of this value
normnoise(Index);
L5=Index%is index of lower 0.85 match

LDiff15=L85-L15%is difference between lower 0.2 and 0.8 matches
LDiff2=L8-L2%is difference between lower 0.2 and 0.8 matches
LDiff3=L7-L3%is difference between lower 0.2 and 0.8 matches

for i=ipeak:Length%searching right side of peak
normnoise2(i,:)=(10^(Ns(i)/10)-10^(avstartnoise/10))/normfact;
endfor
%%normnoise(498)

target = 0.15;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U15=Index

target = 0.2;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U2=Index

target = 0.3;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U3=Index

target = 0.7;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U7=Index

target = 0.8;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U8=Index

target = 0.85;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U85=Index

target = 0.5;%searching for a specific value
Index=find(min(abs(normnoise2-target))==abs(normnoise2-target));%returns index of this value
normnoise2(Index);
U5=Index

UDiff15=U15-U85
UDiff2=U2-U8
UDiff3=U3-U7

UL5=U5-L5




corrazppoint=interval*((azrate*cosd(elevation))^2 + elrate^2)^0.5



Sunnoise_maxYdB=peak-avendnoise

Crude1dBBW =(U8-L8)*corrazppoint% crude1dB beamwidth from full scan
Crude3dBBW=(U5-L5)*corrazppoint% crude HPBW from full scan
joachim_15L=LDiff15*corrazppoint% from first half of scan
joachim_2L=LDiff2*corrazppoint% from first half of scan
joachim_3L=LDiff3*corrazppoint% from first half of scan
joachim_15U=UDiff15*corrazppoint% from first half of scan
joachim_2U=UDiff2*corrazppoint% from first half of scan
joachim_3U=UDiff3*corrazppoint% from first half of scan


Lsidelobelevel_dB=10*log10(normnoisefull(Lsidelobeindex))
Usidelobelevel_dB=10*log10(normnoisefull(Usidelobeindex))

figure(1); % plots in GUI

plot((xaxis),normnoisefull(1:Length)); % plot one cho linear spectrum
line ([0 (length(xaxis))], [0.15 0.15], "linestyle", "-", "color", "r");
line ([0 (length(xaxis))], [0.2 0.2], "linestyle", "-", "color", "r");
line ([0 (length(xaxis))], [0.3 0.3], "linestyle", "-", "color", "r");
line ([0 (length(xaxis))], [0.5 0.5], "linestyle", "-", "color", "r");
line ([0 (length(xaxis))], [0.7 0.7], "linestyle", "-", "color", "r");
line ([0 (length(xaxis))], [0.8 0.8], "linestyle", "-", "color", "r");
line ([0 (length(xaxis))], [0.85 0.85], "linestyle", "-", "color", "r");
%plot (L15,0.15,"g*");
ylim([0.0 1]);
title ("Normalised level");
xlabel ("Data Points");
ylabel ("Level");

grid on;




figure(2); % plots in GUI
hold on
plot((xaxis),Ns(1:Length),'linewidth',3); % plot one cho linear spectrum
plot((xaxis),S(1:Length)); % plot one cho linear spectrum  KA!GT
title ("Raw noise data");
xlabel ("Data Points");
ylabel ("noise power (dB)");
grid on;


#################### KA1GT output ###########################



s1 = strcat("Sun Y factor = ",num2str(Sunnoise_maxYdB), " dB");

s3 = strcat("uncorrected 3dB beamwidth = ",num2str(Crude3dBBW), " deg");
s4 = strcat("sidelobe 1 =  ",num2str(Lsidelobelevel_dB), " dB");
s5 = strcat("sidelobe 2 =  ",num2str(Usidelobelevel_dB), " dB");
s6 = strcat("0.15-0.85 coeff = ",num2str(joachim_15L));
s7 = strcat("0.2-0.8 coeff = ",num2str(joachim_2L));
s8 = strcat("0.3-0.7 coeff = ",num2str(joachim_3L));

waitfor(msgbox({s1,s3,s4,s5,s6,s7,s8}));

close all # close open windows

action = questdlg('Quit or Loop?', 	'What to do next', 	'Quit','Loop','Quit'); ### KA1GT  ### alows looping pack to original filr and running again

                 if (strcmp(action,'Quit'))
                  return
                 endif

endwhile



